@@ -3,7 +3,7 @@ import os
 import sys
 import time
 from inspect import getsource
-from numbers import Number
+from numbers import Integral, Number
 from pathlib import Path
 from tempfile import TemporaryDirectory
 from typing import Callable, Dict, List, Optional, Union
@@ -321,8 +321,9 @@ class FunctionsAPI(APIClient):
         elif self._cognite_client.config.token is not None:
             nonce = _use_token_exchange(self._cognite_client)
 
-        if data:
-            body = {"data": data, "nonce": nonce}
+        if data is None:
+            data = {}
+        body = {"data": data, "nonce": nonce}
         res = self._post(url, json=body)
 
         function_call = FunctionCall._load(res.json(), cognite_client=self._cognite_client)
@@ -490,6 +491,17 @@ def _validate_function_handle(function_handle):
         )
 
 
+def _assert_exactly_one_of_function_id_and_function_external_id(function_id, function_external_id):
+    utils._auxiliary.assert_type(function_id, "function_id", [Integral], allow_none=True)
+    utils._auxiliary.assert_type(function_external_id, "function_external_id", [str], allow_none=True)
+    has_function_id = function_id is not None
+    has_function_external_id = function_external_id is not None
+
+    assert (has_function_id or has_function_external_id) and not (
+        has_function_id and has_function_external_id
+    ), "Exactly one of function_id and function_external_id must be specified"
+
+
 class FunctionCallsAPI(APIClient):
     _LIST_CLASS = FunctionCallList
 
@@ -533,7 +545,7 @@ class FunctionCallsAPI(APIClient):
                 >>> calls = func.list_calls()
 
         """
-        utils._auxiliary.assert_exactly_one_of_id_or_external_id(function_id, function_external_id)
+        _assert_exactly_one_of_function_id_and_function_external_id(function_id, function_external_id)
         if function_external_id:
             function_id = self._cognite_client.functions.retrieve(external_id=function_external_id).id
         filter = {"status": status, "scheduleId": schedule_id, "startTime": start_time, "endTime": end_time}
@@ -570,7 +582,7 @@ class FunctionCallsAPI(APIClient):
                 >>> call = func.retrieve_call(id=2)
 
         """
-        utils._auxiliary.assert_exactly_one_of_id_or_external_id(function_id, function_external_id)
+        _assert_exactly_one_of_function_id_and_function_external_id(function_id, function_external_id)
         if function_external_id:
             function_id = self._cognite_client.functions.retrieve(external_id=function_external_id).id
         resource_path = f"/functions/{function_id}/calls"
@@ -603,7 +615,7 @@ class FunctionCallsAPI(APIClient):
                 >>> response = call.get_response()
 
         """
-        utils._auxiliary.assert_exactly_one_of_id_or_external_id(function_id, function_external_id)
+        _assert_exactly_one_of_function_id_and_function_external_id(function_id, function_external_id)
         if function_external_id:
             function_id = self._cognite_client.functions.retrieve(external_id=function_external_id).id
         url = f"/functions/{function_id}/calls/{call_id}/response"
@@ -639,7 +651,7 @@ class FunctionCallsAPI(APIClient):
                 >>> logs = call.get_logs()
 
         """
-        utils._auxiliary.assert_exactly_one_of_id_or_external_id(function_id, function_external_id)
+        _assert_exactly_one_of_function_id_and_function_external_id(function_id, function_external_id)
         if function_external_id:
             function_id = self._cognite_client.functions.retrieve(external_id=function_external_id).id
         url = f"/functions/{function_id}/calls/{call_id}/logs"
@@ -725,8 +737,9 @@ class FunctionSchedulesAPI(APIClient):
     def create(
         self,
         name: str,
-        function_external_id: str,
         cron_expression: str,
+        function_id: Optional[int] = None,
+        function_external_id: Optional[str] = None,
         client_credentials: Optional[Dict] = None,
         description: str = "",
         data: Optional[Dict] = None,
@@ -735,7 +748,8 @@ class FunctionSchedulesAPI(APIClient):
 
         Args:
             name (str): Name of the schedule.
-            function_external_id (str): External id of the function.
+            function_id (optional, int): Id of the function. This is required if the schedule is created with client_credentials.
+            function_external_id (optional, str): External id of the function. This is deprecated and cannot be used together with client_credentials.
             description (str): Description of the schedule.
             cron_expression (str): Cron expression.
             client_credentials: (optional, Dict): Dictionary containing client credentials:
@@ -754,14 +768,17 @@ class FunctionSchedulesAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> schedule = c.functions.schedules.create(
                     name= "My schedule",
-                    function_external_id="my-external-id",
+                    function_id=123,
                     cron_expression="*/5 * * * *",
                     client_credentials={"client_id": "...", "client_secret": "..."},
                     description="This schedule does magic stuff.")
 
         """
+        _assert_exactly_one_of_function_id_and_function_external_id(function_id, function_external_id)
+
         nonce = None
         if client_credentials:
+            assert function_id is not None, "function_id must be set when creating a schedule with client_credentials."
             nonce = _use_client_credentials(self._cognite_client, client_credentials)
 
         body = {
