@@ -1,8 +1,6 @@
 import os
 from typing import Callable, Dict, List, Optional, Union
 
-from cognite.client._api.datapoints import DatapointsAPI
-from cognite.client._api.files import FilesAPI
 from cognite.client._api_client import APIClient
 from cognite.client.beta import CogniteClient as Client
 
@@ -13,21 +11,13 @@ from cognite.experimental._api.entity_matching import EntityMatchingAPI
 from cognite.experimental._api.functions import FunctionsAPI
 from cognite.experimental._api.integrationruns import IntegrationsRunsAPI
 from cognite.experimental._api.integrations import IntegrationsAPI
+from cognite.experimental._api.match_rules import MatchRulesAPI
 from cognite.experimental._api.model_hosting import ModelHostingAPI
 from cognite.experimental._api.plot_extraction import PlotDataExtractionAPI
 from cognite.experimental._api.pnid_object_detection import PNIDObjectDetectionAPI
-from cognite.experimental._api.pnid_parsing import PNIDParsingAPI
-from cognite.experimental._api.relationships import RelationshipsAPI
-from cognite.experimental._api.schema_completion import SchemaCompletionAPI
+from cognite.experimental._api.pnid_parsing import DiagramsAPI, PNIDParsingAPI
+from cognite.experimental._api.templatecompletion import ExperimentalTemplatesAPI
 from cognite.experimental._api.types import TypesAPI
-from cognite.experimental._api.unstructured import GrepAPI
-
-
-class ExperimentalFilesApi(FilesAPI):
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.unstructured = GrepAPI(self._config, api_version="playground", cognite_client=self)
-
 
 APIClient.RETRYABLE_POST_ENDPOINTS |= {
     "/files/unstructured/search",
@@ -35,7 +25,7 @@ APIClient.RETRYABLE_POST_ENDPOINTS |= {
 }
 APIClient.RETRYABLE_POST_ENDPOINTS |= {
     f"/{api}/{endpoint}"
-    for api in ["types", "labels", "relationships", "functions"]
+    for api in ["types", "labels", "functions", "templates"]
     for endpoint in ["list", "byids", "search"]
 }
 
@@ -103,19 +93,16 @@ class CogniteClient(Client):
             debug=debug,
             **kwargs,
         )
-        # OLD relationships - planned to be removed soon
-        self.relationships_playground = RelationshipsAPI(self._config, api_version="playground", cognite_client=self)
         # NEW assets features - e.g. types
         self.assets_playground = ExperimentalAssetsAPI(self._config, api_version="playground", cognite_client=self)
 
-        self.files = ExperimentalFilesApi(self._config, api_version="v1", cognite_client=self)
         self.model_hosting = ModelHostingAPI(self._config, api_version="playground", cognite_client=self)
         self.types = TypesAPI(self._config, api_version="playground", cognite_client=self)
 
         self.document_parsing = DocumentParsingAPI(self._config, api_version="playground", cognite_client=self)
         self.entity_matching = EntityMatchingAPI(self._config, api_version="playground", cognite_client=self)
+        self.match_rules = MatchRulesAPI(self._config, api_version="playground", cognite_client=self)
         self.pnid_parsing = PNIDParsingAPI(self._config, api_version="playground", cognite_client=self)
-        self.schemas = SchemaCompletionAPI(self._config, api_version="playground", cognite_client=self)
         self.pnid_object_detection = PNIDObjectDetectionAPI(self._config, api_version="playground", cognite_client=self)
         self.annotations = AnnotationsAPI(self._config, api_version="playground", cognite_client=self)
         self.plot_extraction = PlotDataExtractionAPI(self._config, api_version="playground", cognite_client=self)
@@ -123,3 +110,7 @@ class CogniteClient(Client):
         self.functions = FunctionsAPI(self.config, api_version="playground", cognite_client=self)
         self.integrations = IntegrationsAPI(self._config, api_version="playground", cognite_client=self)
         self.integration_runs = IntegrationsRunsAPI(self._config, api_version="playground", cognite_client=self)
+
+        self.diagrams = DiagramsAPI(self._config, api_version=self._API_VERSION, cognite_client=self)
+        # template completion only
+        self.templates = ExperimentalTemplatesAPI(self._config, api_version=self._API_VERSION, cognite_client=self)
