@@ -10,6 +10,7 @@ from cognite.client.data_classes._base import (
     CogniteResourceList,
     CogniteUpdate,
 )
+from cognite.client.utils._auxiliary import to_camel_case
 
 from cognite.experimental.data_classes.utils.pandas import dataframe_summarize
 from cognite.experimental.data_classes.utils.rules_output import _color_matches, _label_groups
@@ -22,6 +23,10 @@ class EntityMatchingMatchRule(CogniteResource):
         extractors=None,
         priority=None,
         matches=None,
+        flags=None,
+        conflicts=None,
+        overlaps=None,
+        number_of_matches=None,
         num_conflicts=None,
         num_overlaps=None,
         cognite_client=None,
@@ -30,6 +35,10 @@ class EntityMatchingMatchRule(CogniteResource):
         self.extractors = extractors
         self.priority = priority
         self.matches = matches
+        self.flags = flags
+        self.conflicts = conflicts
+        self.overlaps = overlaps
+        self.number_of_matches = number_of_matches
         self.num_conflicts = num_conflicts
         self.num_overlaps = num_overlaps
         self._cognite_client = cognite_client
@@ -37,12 +46,13 @@ class EntityMatchingMatchRule(CogniteResource):
     def _repr_html_(self):
         extractors = _label_groups(copy.deepcopy(self.extractors), self.conditions)
 
-        info_df = super().to_pandas(camel_case=True).drop(["extractors", "conditions"])
+        info_df = super().to_pandas(camel_case=True)
         info_df = dataframe_summarize(info_df)
         info_df.index.name = "Stats"
-
-        coloured_matches_table = _color_matches(extractors, self.matches)
-        return info_df._repr_html_() + coloured_matches_table
+        html = info_df._repr_html_()
+        if self.matches is not None:
+            html += _color_matches(extractors, self.matches)
+        return html
 
 
 class EntityMatchingMatchRuleList(CogniteResourceList):
@@ -50,9 +60,11 @@ class EntityMatchingMatchRuleList(CogniteResourceList):
     _ASSERT_CLASSES = False
 
     def _repr_html_(self):
+        if not self.data:
+            return super()._repr_html_()
         try:
             from ipywidgets import interact  # dont want this as dependency
-        except ImportError:
+        except (ImportError, ModuleNotFoundError):
             return super()._repr_html_()
 
         @interact(rule_number=(0, len(self) - 1))
@@ -136,9 +148,9 @@ class EntityMatchingPipelineRun(ContextualizationJob):
         self._pipeline = value
 
     @property
-    def suggested_rules(self):
-        """(deprecated) List of suggested old-style rules. Depends on .result and may block"""
-        return self.result["suggestedRules"]
+    def errors(self):
+        """Returns list of error messages encountered while running. Depends on .result and may block"""
+        return self.result.get("errors")
 
     @property
     def generated_rules(self):
@@ -146,19 +158,19 @@ class EntityMatchingPipelineRun(ContextualizationJob):
         return EntityMatchingMatchRuleList._load(self.result["generatedRules"], cognite_client=self._cognite_client)
 
     @property
-    def matches(self) -> EntityMatchingMatchRuleList:
+    def matches(self) -> EntityMatchingMatchList:
         """List of matches. Depends on .result and may block"""
         matches = self.result["matches"]
-        match_fields = self.pipeline.model_parameters.get("matchFields", [{"source": "name", "target": "name"}])
+        match_fields = (self.pipeline.model_parameters or {}).get("matchFields", [{"source": "name", "target": "name"}])
         for match in matches:
             match["match_fields"] = match_fields  # for nice output
         return EntityMatchingMatchList._load(matches, cognite_client=self._cognite_client)
 
     def _repr_html_(self):
+        result = self.result  # TODO: optional loading? do this first for now for consistent status
         df = super().to_pandas()
-        # TODO: optional loading?
-        df.loc["matches"] = [self.result["matches"]]
-        df.loc["generatedRules"] = [self.result["generatedRules"]]
+        df.loc["matches"] = [result["matches"]]
+        df.loc["generatedRules"] = [result["generatedRules"]]
         return dataframe_summarize(df)._repr_html_()
 
 
@@ -169,6 +181,23 @@ class EntityMatchingPipelineRunList(CogniteResourceList):
 
 
 class EntityMatchingPipeline(CogniteResource):
+    """Entity matching pipeline, used to continuously iterate and improve an entity matching model.
+       The fields below can be filled when creating a pipeline. Other fields should be left empty, and return status information on successful creation and retrieval.
+
+    Args:
+        external_id, name, description: standard fields for a resource.
+        model_parameters: a dictionary with fields `match_fields`, `feature_type`, `classifier`, as in the `fit` method for entity matching.
+        sources, targets: a dictionary of the format {'resource': ..., 'dataSetIds': [{'id':...},{'externalId':...}]}
+        true_matches: existing matches with reasonable certainty to use in training.
+        confirmed_matches: user-confirmed certain matches which will be used to override any other results.
+        rejected_matches: user-confirmed wrong results which will be used to blank output for a match result if it is one of these.
+        use_existing_matches: If set, uses existing matches on resources as additional true_matches (but not confirmed_matches).
+        replacements: Expects a list of {'field':.., 'string':.. ,'replacement': ..} which will be used to replace substrings in a field with a synonym, such as "Pressure Transmitter" -> "PT", or "Æ" -> AE. Field can be '*' for all.
+        relationships_label: If set, writes relationships with this label to the tenant (along with a pipeline-specific and general entity matching label). Requires whitelisting by auth.
+        rules: list of matching rules (either old or new format)
+        schedule_interval: automatically schedule pipeline to be run every this many seconds.
+    """
+
     _RESOURCE_PATH = "/context/entitymatching/pipelines"
     _STATUS_PATH = _RESOURCE_PATH + "/"
 
@@ -185,6 +214,7 @@ class EntityMatchingPipeline(CogniteResource):
         rejected_matches: List = None,
         confirmed_matches: List = None,
         use_existing_matches: bool = None,
+        replacements: List[Dict] = None,
         relationships_label: str = None,
         score_threshold: float = None,
         schedule_interval: int = None,
@@ -196,21 +226,6 @@ class EntityMatchingPipeline(CogniteResource):
         status_time=None,
         cognite_client=None,
     ):
-        """
-               The fields below can be filled when creating a pipeline. Other fields should be left empty, and return status information on successful creation and retrieval.
-               Args:
-                   external_id, name, description: standard fields for a resource.
-                   model_parameters: a dictionary with fields `match_fields`, `feature_type`, `classifier`, as in the `fit` method for entity matching.
-                   sources, targets: a dictionary of the format {'resource': ..., 'dataSetIds': [{'id':...},{'externalId':...}]}
-                   true_matches: existing matches with reasonable certainty to use in training.
-                   confirmed_matches: user-confirmed certain matches which will be used to override any other results.
-                   rejected_matches: user-confirmed wrong results which will be used to blank output for a match result if it is one of these.
-                   use_existing_matches: If set, uses existing matches on resources as additional true_matches (but not confirmed_matches).
-                   relationships_label: If set, writes relationships with this label to the tenant (along with a pipeline-specific and general entity matching label). Requires whitelisting by auth.
-                   rules: list of matching rules (either old or new format)
-        (either old or new format)
-                   schedule_interval: automatically schedule pipeline to be run every this many seconds.
-        """
 
         self.id = id
         self.external_id = external_id
@@ -223,6 +238,7 @@ class EntityMatchingPipeline(CogniteResource):
         self.confirmed_matches = confirmed_matches
         self.rejected_matches = rejected_matches
         self.use_existing_matches = use_existing_matches
+        self.replacements = replacements
         self.relationships_label = relationships_label
         self.score_threshold = score_threshold
         self.rules = rules
@@ -237,25 +253,31 @@ class EntityMatchingPipeline(CogniteResource):
         self._cognite_client = cognite_client
 
     def run(self) -> EntityMatchingPipelineRun:
+        """Runs the pipeline and returns a run job"""
         return self._cognite_client.entity_matching.pipelines.run(id=self.id)
 
     def runs(self) -> EntityMatchingPipelineRunList:
+        """Retrieve the list of runs"""
         return self._cognite_client.entity_matching.pipelines.runs.list(id=self.id)
 
     def latest_run(self) -> EntityMatchingPipelineRun:
+        """Retrieve the latest run"""
         return self._cognite_client.entity_matching.pipelines.runs.retrieve_latest(id=self.id)
 
     def to_pandas(self, camel_case=False):
         df = dataframe_summarize(super().to_pandas(camel_case=camel_case))
         # expand
         for f in ["sources", "targets", "model_parameters"]:
-            for k, v in (getattr(self, f) or {}).items():
+            for k, v in (getattr(self, f, {}) or {}).items():
                 if isinstance(v, list):
                     for i, vi in enumerate(v):
                         df.loc[f"{f}.{k}[{i}]"] = [vi]
                 else:
                     df.loc[f"{f}.{k}"] = [v]
-            df = df.drop(f)
+            if camel_case:
+                f = to_camel_case(f)
+            if f in df.index:
+                df = df.drop(f)
         return df
 
 
@@ -307,6 +329,10 @@ class EntityMatchingPipelineUpdate(CogniteUpdate):  # not implemented yet
     def rules(self):
         return EntityMatchingPipelineUpdate._PrimitiveUpdate(self, "rules")
 
+    @property
+    def replacements(self):
+        return EntityMatchingPipelineUpdate._PrimitiveUpdate(self, "replacements")
+
     @property
     def score_threshold(self):
         return EntityMatchingPipelineUpdate._PrimitiveUpdate(self, "scoreThreshold")
@@ -321,12 +347,50 @@ class EntityMatchingPipelineList(CogniteResourceList):
     _UPDATE = EntityMatchingPipelineUpdate
 
 
+class MatchRulesSuggestJob(ContextualizationJob):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self._status_path = "/context/matchrules/suggest/"
+
+    @property
+    def rules(self) -> EntityMatchingMatchRuleList:
+        """Depends on .result and may block"""
+        return EntityMatchingMatchRuleList._load(self.result["rules"])
+
+    def _repr_html_(self):
+        rules = self.rules  # TODO: optional loading? before super() for status
+        df = super().to_pandas()
+        df.loc["rules"] = [f"{len(rules)} items"]
+        return df._repr_html_()
+
+
+class MatchRulesApplyJob(ContextualizationJob):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self._status_path = "/context/matchrules/suggest/"
+
+    @property
+    def rules(self) -> EntityMatchingMatchRuleList:
+        """Depends on .result and may block"""
+        reformated_rules = [{**item["rule"], **item} for item in self.result["items"]]
+        for rule in reformated_rules:
+            rule.pop("rule")
+        return EntityMatchingMatchRuleList._load(reformated_rules)
+
+    def _repr_html_(self):
+        rules = self.rules  # TODO: optional loading? before super() for status
+        df = super().to_pandas()
+        df.loc["rules"] = [f"{len(rules)} items"]
+        return df._repr_html_()
+
+
 class PNIDDetection(CogniteResource):
-    def __init__(self, text=None, type=None, confidence=None, bounding_box=None, cognite_client=None):
+    def __init__(self, text=None, type=None, confidence=None, bounding_box=None, entities=None, cognite_client=None):
         self.text = text
         self.type = type
         self.confidence = confidence
         self.bounding_box = bounding_box
+        self.entities = entities
         self._cognite_client = cognite_client
 
 
@@ -347,7 +411,7 @@ class PNIDDetectionList(CogniteResourceList):
             from bounding_box import bounding_box as bb
             from pdf2image import convert_from_bytes
             from PIL import Image
-        except ImportError as e:
+        except (ImportError, ModuleNotFoundError) as e:
             warnings.warn(
                 f"Module {e.name} missing, 'pip install Pillow numpy bounding_box pdf2image' for advanced visualization of results"
             )
@@ -376,6 +440,11 @@ class PNIDDetectionList(CogniteResourceList):
         except Exception:
             return None
 
+    @classmethod
+    def _load(cls, resource_list: Union[List, str], cognite_client=None):
+        loaded = super()._load(resource_list, cognite_client)
+        return loaded
+
 
 class PNIDDetectionPageList(UserList):
     def __init__(self, data, file_id):
@@ -405,12 +474,15 @@ class PNIDConvertResults(ContextualizationJob):
 
 
 class PNIDDetectResults(ContextualizationJob):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
     @property
     def matches(self) -> PNIDDetectionList:
         """Returns detected items"""
-        return PNIDDetectionList._load(self.result["items"], cognite_client=self._cognite_client)
+        return PNIDDetectionList._load(self.result["items"], cognite_client=self._cognite_client,)
 
-    def to_pandas(self, camel_case=False):
+    def to_pandas(self, camel_case: bool = False):
         df = super().to_pandas(camel_case=camel_case)
         df.loc["matches"] = f"{len(self.matches)} items"
         return df
@@ -444,3 +516,99 @@ class PNIDDetectResults(ContextualizationJob):
     def ocr(self) -> PNIDDetectionPageList:
         """Retrieve raw OCR results, for example, to visualize"""
         return self._cognite_client.pnid_parsing.ocr(file_id=self.file_id)
+
+
+class DiagramConvertPage(CogniteResource):
+    def __init__(self, page=None, png_url=None, svg_url=None, cognite_client=None):
+        self.page = page
+        self.png_url = png_url
+        self.svg_url = svg_url
+        self._cognite_client = cognite_client
+
+
+class DiagramConvertPageList(CogniteResourceList):
+    _RESOURCE = DiagramConvertPage
+    _ASSERT_CLASSES = False
+
+
+class DiagramConvertItem(CogniteResource):
+    def __init__(self, file_id=None, file_external_id=None, results=None, cognite_client=None):
+        self.file_id = file_id
+        self.file_external_id = file_external_id
+        self.results = results
+        self._cognite_client = cognite_client
+
+    def __len__(self):
+        return len(self.results)
+
+    @property
+    def pages(self):
+        return DiagramConvertPageList._load(self.results, cognite_client=self._cognite_client)
+
+    def to_pandas(self, camel_case: bool = False):
+        df = super().to_pandas(camel_case=camel_case)
+        df.loc["results"] = f"{len(df['results'])} pages"
+        return df
+
+
+class DiagramConvertResults(ContextualizationJob):
+    def __getitem__(self, find_id) -> DiagramConvertItem:
+        """retrieves the results for the file with (external) id"""
+        found = [
+            item
+            for item in self.result["items"]
+            if item.get("fileId") == find_id or item.get("fileExternalId") == find_id
+        ]
+        if not found:
+            raise IndexError(f"File with (external) id {find_id} not found in results")
+        if len(found) != 1:
+            raise IndexError(f"Found multiple results for file with (external) id {find_id}, use .items instead")
+        return DiagramConvertItem._load(found[0], cognite_client=self._cognite_client)
+
+    @property
+    def items(self) -> List[DiagramConvertItem]:
+        """returns a list of all results by file"""
+        return [DiagramConvertItem._load(item, cognite_client=self._cognite_client) for item in self.result["items"]]
+
+
+class DiagramDetectItem(CogniteResource):
+    def __init__(self, file_id=None, file_external_id=None, annotations=None, error_message=None, cognite_client=None):
+        self.file_id = file_id
+        self.file_external_id = file_external_id
+        self.annotations = annotations
+        self.error_message = error_message
+        self._cognite_client = cognite_client
+
+    def to_pandas(self, camel_case: bool = False):
+        df = super().to_pandas(camel_case=camel_case)
+        df.loc["annotations"] = f"{len(df['annotations'])} annotations"
+        return df
+
+
+class DiagramDetectResults(ContextualizationJob):
+    def __getitem__(self, find_id) -> DiagramDetectItem:
+        """retrieves the results for the file with (external) id"""
+        found = [
+            item
+            for item in self.result["items"]
+            if item.get("fileId") == find_id or item.get("fileExternalId") == find_id
+        ]
+        if not found:
+            raise IndexError(f"File with (external) id {find_id} not found in results")
+        if len(found) != 1:
+            raise IndexError(f"Found multiple results for file with (external) id {find_id}, use .items instead")
+        return DiagramDetectItem._load(found[0], cognite_client=self._cognite_client)
+
+    @property
+    def items(self) -> List[DiagramDetectItem]:
+        """returns a list of all results by file"""
+        return [DiagramDetectItem._load(item, cognite_client=self._cognite_client) for item in self.result["items"]]
+
+    @property
+    def errors(self) -> List[str]:
+        """returns a list of all error messages across files"""
+        return [item["errorMessage"] for item in self.result["items"] if "errorMessage" in item]
+
+    def convert(self) -> DiagramConvertResults:
+        """Convert a P&ID to an interactive SVG where the provided annotations are highlighted"""
+        return self._cognite_client.diagrams.convert(detect_job=self)
